@@ -1,5 +1,5 @@
 /* grep.c - main driver file for grep.
-   Copyright (C) 1992, 1997-2002, 2004-2020 Free Software Foundation, Inc.
+   Copyright (C) 1992, 1997-2002, 2004-2021 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -128,8 +128,9 @@ hash_pattern (void const *pat, size_t n_buckets)
 {
   size_t h = 0;
   intptr_t pat_offset = (intptr_t) pat - 1;
-  for (char const *s = pattern_array + pat_offset; *s != '\n'; s++)
-    h = *s + ((h << 9) | (h >> (SIZE_WIDTH - 9)));
+  unsigned char const *s = (unsigned char const *) pattern_array + pat_offset;
+  for ( ; *s != '\n'; s++)
+    h = h * 33 ^ *s;
   return h % n_buckets;
 }
 static bool _GL_ATTRIBUTE_PURE
@@ -447,7 +448,7 @@ fputs_errno (char const *s)
     stdout_errno = errno;
 }
 
-static void _GL_ATTRIBUTE_FORMAT_PRINTF (1, 2)
+static void _GL_ATTRIBUTE_FORMAT_PRINTF_STANDARD (1, 2)
 printf_errno (char const *format, ...)
 {
   va_list ap;
@@ -641,7 +642,7 @@ typedef size_t (*execute_fp_t) (void *, char const *, size_t, size_t *,
 static execute_fp_t execute;
 static void *compiled_pattern;
 
-static char const *
+char const *
 input_filename (void)
 {
   if (!filename)
@@ -879,9 +880,9 @@ enum { INITIAL_BUFSIZE = 96 * 1024 };
 /* Return VAL aligned to the next multiple of ALIGNMENT.  VAL can be
    an integer or a pointer.  Both args must be free of side effects.  */
 #define ALIGN_TO(val, alignment) \
-  ((size_t) (val) % (alignment) == 0 \
+  ((uintptr_t) (val) % (alignment) == 0 \
    ? (val) \
-   : (val) + ((alignment) - (size_t) (val) % (alignment)))
+   : (val) + ((alignment) - (uintptr_t) (val) % (alignment)))
 
 /* Add two numbers that count input bytes or lines, and report an
    error if the addition overflows.  */
@@ -942,15 +943,8 @@ fillbuf (size_t save, struct stat const *st)
   char *readbuf;
   size_t readsize;
 
-  /* Offset from start of buffer to start of old stuff
-     that we want to save.  */
-  size_t saved_offset = buflim - save - buffer;
-
   if (pagesize <= buffer + bufalloc - sizeof (uword) - buflim)
-    {
-      readbuf = buflim;
-      bufbeg = buflim - save;
-    }
+    readbuf = buflim;
   else
     {
       size_t minsize = save + pagesize;
@@ -988,9 +982,8 @@ fillbuf (size_t save, struct stat const *st)
 
       newbuf = bufalloc < newalloc ? xmalloc (bufalloc = newalloc) : buffer;
       readbuf = ALIGN_TO (newbuf + 1 + save, pagesize);
-      bufbeg = readbuf - save;
-      memmove (bufbeg, buffer + saved_offset, save);
-      bufbeg[-1] = eolbyte;
+      size_t moved = save + 1;  /* Move the preceding byte sentinel too.  */
+      memmove (readbuf - moved, buflim - moved, moved);
       if (newbuf != buffer)
         {
           free (buffer);
@@ -998,6 +991,8 @@ fillbuf (size_t save, struct stat const *st)
         }
     }
 
+  bufbeg = readbuf - save;
+
   clear_asan_poison ();
 
   readsize = buffer + bufalloc - sizeof (uword) - readbuf;
@@ -2047,6 +2042,8 @@ Context control:\n\
 "));
       printf (_("\
   -NUM                      same as --context=NUM\n\
+      --group-separator=SEP  print SEP on line between matches with context\n\
+      --no-group-separator  do not print separator for matches with context\n\
       --color[=WHEN],\n\
       --colour[=WHEN]       use markers to highlight the matching strings;\n\
                             WHEN is 'always', 'never', or 'auto'\n\
@@ -2106,66 +2103,6 @@ setmatcher (char const *m, int matcher)
   die (EXIT_TROUBLE, 0, _("invalid matcher %s"), m);
 }
 
-/* Find the white-space-separated options specified by OPTIONS, and
-   using BUF to store copies of these options, set ARGV[0], ARGV[1],
-   etc. to the option copies.  Return the number N of options found.
-   Do not set ARGV[N] to NULL.  If ARGV is NULL, do not store ARGV[0]
-   etc.  Backslash can be used to escape whitespace (and backslashes).  */
-static size_t
-prepend_args (char const *options, char *buf, char **argv)
-{
-  char const *o = options;
-  char *b = buf;
-  size_t n = 0;
-
-  for (;;)
-    {
-      while (c_isspace (to_uchar (*o)))
-        o++;
-      if (!*o)
-        return n;
-      if (argv)
-        argv[n] = b;
-      n++;
-
-      do
-        if ((*b++ = *o++) == '\\' && *o)
-          b[-1] = *o++;
-      while (*o && ! c_isspace (to_uchar (*o)));
-
-      *b++ = '\0';
-    }
-}
-
-/* Prepend the whitespace-separated options in OPTIONS to the argument
-   vector of a main program with argument count *PARGC and argument
-   vector *PARGV.  Return the number of options prepended.  */
-static int
-prepend_default_options (char const *options, int *pargc, char ***pargv)
-{
-  if (options && *options)
-    {
-      char *buf = xmalloc (strlen (options) + 1);
-      size_t prepended = prepend_args (options, buf, NULL);
-      int argc = *pargc;
-      char *const *argv = *pargv;
-      char **pp;
-      enum { MAX_ARGS = MIN (INT_MAX, SIZE_MAX / sizeof *pp - 1) };
-      if (MAX_ARGS - argc < prepended)
-        xalloc_die ();
-      pp = xmalloc ((prepended + argc + 1) * sizeof *pp);
-      *pargc = prepended + argc;
-      *pargv = pp;
-      *pp++ = *argv++;
-      pp += prepend_args (options, buf, pp);
-      while ((*pp++ = *argv++))
-        continue;
-      return prepended;
-    }
-
-  return 0;
-}
-
 /* Get the next non-digit option from ARGC and ARGV.
    Return -1 if there are no more options.
    Process any digit options that were encountered on the way,
@@ -2530,7 +2467,7 @@ main (int argc, char **argv)
   char *keys = NULL;
   size_t keycc = 0, keyalloc = 0;
   int matcher = -1;
-  int opt, prepended;
+  int opt;
   int prev_optind, last_recursive;
   int fread_errno;
   intmax_t default_context;
@@ -2574,11 +2511,6 @@ main (int argc, char **argv)
   if (!pattern_table)
     xalloc_die ();
 
-  prepended = prepend_default_options (getenv ("GREP_OPTIONS"), &argc, &argv);
-  if (prepended)
-    error (0, 0, _("warning: GREP_OPTIONS is deprecated;"
-                   " please use an alias or script"));
-
   while (prev_optind = optind,
          (opt = get_nondigit_option (argc, argv, &default_context)) != -1)
     switch (opt)
@@ -2644,8 +2576,8 @@ main (int argc, char **argv)
         break;
 
       case 'u':
-        /* Obsolete option; it has no effect.  FIXME: Diagnose use of
-           this option starting in (say) the year 2020.  */
+        /* Obsolete option; it had no effect; FIXME: remove in 2023  */
+        error (0, 0, _("warning: --unix-byte-offsets (-u) is obsolete"));
         break;
 
       case 'V':
@@ -3056,7 +2988,7 @@ main (int argc, char **argv)
     {
       files = argv + optind;
     }
-  else if (directories == RECURSE_DIRECTORIES && prepended < last_recursive)
+  else if (directories == RECURSE_DIRECTORIES && 0 < last_recursive)
     {
       static char *const cwd_only[] = { (char *) ".", NULL };
       files = cwd_only;
